#  Created by Daniel Nazarian on 2/22/22, 3:34 PM
#  Copyright (c) Daniel Nazarian. 2022 . All rights reserved.
#
#  Please do NOT use, edit, distribute, or otherwise use this code without consent.
#  For questions, comments, concerns, and more -> dnaz@danielnazarian.com
#
#  Please do NOT use, edit, distribute, or otherwise use this code without consent.
#  For questions, comments, concerns, and more -> dnaz@danielnazarian.com
#
#  Please do NOT use, edit, distribute, or otherwise use this code without consent.
#  For questions, comments, concerns, and more -> dnaz@danielnazarian.com



from django.core.exceptions import ValidationError
from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated

from ..helpers import api_response_success, api_response_error
from ..models.email import NotificationEmail
from ..serializers.email import NotificationEmailSerializer

"""
============================================================================================ #
EMAIL ====================================================================================== #
============================================================================================ #
"""


# =============================== #
#
# NOTIFICATION EMAIL VIEW SET
#
class NotificationEmailViewSet(viewsets.GenericViewSet):
    queryset = NotificationEmail.objects.all()
    serializer_class = NotificationEmailSerializer
    permission_classes = (IsAuthenticated,)

    def list(self, request, *args, **kwargs):
        """
        Endpoint to list out Email Notifications
        """
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(
            self.filter_queryset(
                self.queryset.filter(recipients__contains=request.user.email)
            ),
            many=True,
            context={"request": request},
        )
        page = self.paginate_queryset(serializer.data)
        return self.get_paginated_response(page)

    def retrieve(self, request, *args, **kwargs):
        """
        Endpoint to retrieve specific Email Notification

        :param uuid pk: UUID of NotificationEmail to retrieve
        """
        pk = self.kwargs.get("pk")
        try:
            notification_email = NotificationEmail.objects.get(pk=pk)
            if not notification_email.recipients_contains(request.user.email):
                raise NotificationEmail.DoesNotExist
        except (NotificationEmail.DoesNotExist, ValidationError):
            return api_response_error("Notification not found.")

        # serializer and return
        serializer = self.get_serializer_class()(
            notification_email, context={"request": request}
        )
        return api_response_success(serializer.data)



from threading import Thread


#
# EmailThread
#
class EmailThread(Thread):
    """
    Simple email thread to send emails asynchronously.
    Simply pass a function you want to run in a thread and any arguments
    """

    def run(self, func, *args):
        func(*args)



from django.contrib.auth import get_user_model

from django.test import TestCase


"""
# ========================================================================= #
# BASE MODEL TEST CASE ==================================================== #
# ========================================================================= #
"""


class BaseModelTestCase(TestCase):
    username = "testuser1"
    base_email = "test@email.com"
    base_password = "12345"

    def setUp(self):
        super(TestCase, self).setUp()

        # USER 1 ====================================== #
        self.base_user = get_user_model().objects.create_user(
            username=self.username, email=self.base_email, password=self.base_password
        )
        self.client.force_login(self.base_user)



from ..base import BaseModelTestCase
from ....models.email import NotificationEmailTemplate

"""
# ========================================================================= #
# TEST NOTIFICATION EMAIL TEMPLATE MANAGER ================================ #
# ========================================================================= #
"""


class TestNotificationEmailTemplateManager(BaseModelTestCase):
    email = "test+1@example.com"
    password = "password"

    def setUp(self):
        super(TestNotificationEmailTemplateManager, self).setUp()

    # =================================================================== #
    # BASIC TESTS ======================================================= #
    # =================================================================== #

    def test_template_exists_by_path(self):
        template = "django-dans-emails/default.html"
        email_template = NotificationEmailTemplate.objects.find_email_template(template)
        self.assertEqual(email_template.path, template)

    def test_template_exists_by_path_without_emails(self):
        template = "default.html"
        email_template = NotificationEmailTemplate.objects.find_email_template(template)
        self.assertEqual(email_template.path, template)

    def test_template_exists_by_nickname(self):
        template = "nickname1"
        NotificationEmailTemplate.objects.create(nickname=template)
        email_template = NotificationEmailTemplate.objects.find_email_template(template)
        self.assertEqual(email_template.nickname, template)

    def test_template_does_not_exists_by_nickname(self):
        template = "nickname1"
        email_template = NotificationEmailTemplate.objects.find_email_template(template)
        self.assertEqual(email_template, None)

    def test_template_does_not_exists_by_path(self):
        template = "django-dans-emails/invalid"
        email_template = NotificationEmailTemplate.objects.find_email_template(template)
        self.assertEqual(email_template, None)



from ..base import (
    BaseModelTestCase,
)
from ....models.email import NotificationEmail, NotificationEmailTemplate

"""
# ========================================================================= #
# TEST EMAIL NOTIFICATION ================================================= #
# ========================================================================= #
"""


class TestEmailNotification(BaseModelTestCase):
    model = NotificationEmail

    def setUp(self):
        self.email_template_nickname = "template1"
        self.email_template = NotificationEmailTemplate.objects.create(
            nickname=self.email_template_nickname,
        )
        super(TestEmailNotification, self).setUp()

    # =================================================================== #
    # BASIC TESTS ======================================================= #
    # =================================================================== #

    def test_str(self):
        self.notification = self.model.objects.create(
            template=self.email_template,
            recipients=self.base_email,
            sender=self.base_email,
        )
        self.assertNotEqual(str(self.notification), None)

    def test_with_subject(self):
        subject = "this is a test subject"
        self.notification = self.model.objects.create(
            recipients=self.base_email, sender=self.base_email, subject=subject
        )

        self.assertEqual(self.notification.subject, subject)

    def test_with_context(self):
        context = {"user": "213542465346"}
        self.notification = self.model.objects.create(
            recipients=self.base_email, sender=self.base_email, context=context
        )

        self.assertEqual(self.notification.context, context)



from ..base import (
    BaseModelTestCase,
)
from ....models.push import NotificationPush

"""
# ========================================================================= #
# TEST PUSH NOTIFICATION ================================================== #
# ========================================================================= #
"""


class TestPushNotification(BaseModelTestCase):
    model = NotificationPush

    def setUp(self):
        super(TestPushNotification, self).setUp()

    # =================================================================== #
    # BASIC TESTS ======================================================= #
    # =================================================================== #

    def test_str(self):
        self.notification = self.model.objects.create(
            recipients=self.base_email, sender=self.base_email
        )

        self.assertNotEqual(str(self.notification), None)

    def test_with_message(self):
        message = "this is a test message"
        self.notification = self.model.objects.create(
            recipients=self.base_email, sender=self.base_email, message=message
        )

        self.assertEqual(self.notification.message, message)



from rest_framework import serializers

from .base import BaseSerializer
from ..models.email import NotificationEmail, NotificationEmailTemplate

"""
============================================================================================ #
EMAIL SERIALIZERS ========================================================================== #
============================================================================================ #
"""


#
# NOTIFICATION EMAIL TEMPLATE ========================== #
#
class NotificationEmailTemplateSerializer(BaseSerializer):
    class Meta:
        model = NotificationEmailTemplate
        fields = (
            "id",
            "path",
            "nickname",
        )


#
# NOTIFICATION EMAIL ========================== #
#
class NotificationEmailSerializer(BaseSerializer):
    template_ref = NotificationEmailTemplateSerializer(
        source="template", required=False
    )
    content = serializers.SerializerMethodField()

    class Meta:
        model = NotificationEmail
        fields = (
            "id",
            "template",
            "template_ref",
            "subject",
            "context",
            "datetime_created",
            "datetime_sent",
            "sent_successfully",
            "sender",
            "recipients",
            "content",
        )
        read_only_fields = (
            "id",
            "template_ref",
            "datetime_created",
        )

    @staticmethod
    def get_content(obj):
        return obj.template.html_to_str(obj.context)



from django.db.models import Q

from .models import NotificationEmail, NotificationBasic, NotificationPush

"""
============================================================================================ #
NOTIFICATION MANAGER ======================================================================= #
============================================================================================ #
"""


#
# Generic manager for notification models
#
# Not sure if this will be useful or necessary, can be used to enforce ownership
# on list/retrieval function.
#
# Ideally all CRUD actions on notifications would be done through this to make it a
# bit simpler but that would be a lot of parameter drilling at the moment.
#
class NotificationManager:
    @staticmethod
    def query_ownership(user_email):
        return Q(sender__contains=user_email) | Q(recipients__contains=user_email)

    #
    # RETRIEVE
    #
    def get_notifications_email(self, user_email):
        """
        Get all the EMAIL notifications associated with this user
        :param str user_email: user email to search

        :returns: NotificationEmail queryset
        """
        return NotificationEmail.objects.filter(self.query_ownership(user_email))

    def get_notifications_basic(self, user_email):
        """
        Get all the BASIC notifications associated with this user
        :param str user_email: user email to search

        :returns: NotificationBasic queryset
        """
        return NotificationBasic.objects.filter(self.query_ownership(user_email))

    def get_notifications_push(self, user_email):
        """
        Get all the PUSH notifications associated with this user
        :param str user_email: user email to search

        :returns: NotificationPush queryset
        """
        return NotificationPush.objects.filter(self.query_ownership(user_email))

    def get_notifications_all(self, user_email):
        """
        Get all the notifications associated with this user
        :param str user_email: user email to search

        :returns: dict containing all this users different types of notifications
        :rtype: {'emails': emails, 'basic': basic, 'push': push}
        """
        emails = self.get_notifications_email(user_email=user_email)
        basic = self.get_notifications_basic(user_email=user_email)
        push = self.get_notifications_push(user_email=user_email)
        return {"emails": emails, "basic": basic, "push": push}

    #
    # UPDATE
    #
    @staticmethod
    def mark_notification_basic_read(notification_basic, read=True):
        """
        Mark a NotificationBasic as read
        :param NotificationBasic notification_basic: notification basic to update
        :param bool read: mark read or not
        """
        notification_basic.read = read
        notification_basic.save()
